<?php
/**
 * Log Logic
 * @package     Log
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Log_Logic
{
    /** ログディレクトリ */
    const LOG_DIR = '/log';
    /** ログファイル拡張子 */
    const LOG_EXT = 'log';

    /**
     * コンストラクタ
     */
    public function __construct()
    {
        
    }

    /**
     * ログビューワー画面
     * @return array $log_file_list
     */
    public function viewer()
    {
        $log_file_list = $this->get_log_file_list();

        return $log_file_list;
    }

    /**
     * ログ出力
     * ログファイルの内容をテキストとして返す
     * @param string $file_name
     * @return boolean 
     */
    public function output_log($file_name)
    {
        $log_file_list = $this->get_log_file_list();
        if (array_search($file_name, $log_file_list) === false) {
            // ログディレクトリ内にファイルがない場合
            return false;
        }
        $file_path = DATA_ROOT . self::LOG_DIR . '/' . $file_name . '.' . self::LOG_EXT;

        header('Content-type: text/plain; charset=UTF-8');
        readfile($file_path);
        return true;
    }

    /**
     * ログファイル一覧を配列で返す
     * @return array $log_file_list
     */
    private function get_log_file_list()
    {
        $log_file_list = array();
        // ログディレクトリパス
        $log_dir = DATA_ROOT . self::LOG_DIR;

        // ログディレクトリ内のファイルとディレクトリのリスト
        $file_list = scandir($log_dir);

        // ログファイルのみを取り出す
        foreach ($file_list as $file_name) {
            // .で始まるファイルは除外。
            if (substr($file_name, 0, 1) === '.') {
                continue;
            }
            $path_parts = pathinfo($file_name);
            // 拡張子 .log だけにする
            if ($path_parts['extension'] === self::LOG_EXT) {
                $log_file_list[] = str_replace('.' . self::LOG_EXT, '', $path_parts['basename']);
            }
        }
        // 最新順（高位から低位に）に並び替える
        rsort($log_file_list);

        return $log_file_list;
    }
}