<?php
/**
 * Main Logic
 * @package     Main
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Main_Logic
{

    /**
     * コンストラクタ
     */
    public function __construct()
    {

    }

    /**
     * サイトビューで開いているページの情報を返す
     * @param string $pathname
     * @return array
     */
    public function get_info($pathname)
    {
        // SITE_PATH が指定されているばあい
        if (strlen(SITE_PATH) > 0) {
            // SITE_PATH下であることを確認
            if (strpos($pathname, SITE_PATH) !== 0) {
                // 管理対象ディレクトリ配下でなければ空を返す
                return array(array(), array(), '');
            }
            // SITE_PATH部分を省く
            $pathname = substr($pathname, strlen(SITE_PATH));
        }

        // ページ情報取得
        $page_dao = new Page_Dao();
        $page = $page_dao->select_by_page_path($pathname);
        if ($page === null && substr($pathname, (strlen($pathname) - 1)) === '/') {
            $directory_index = Me_Config::get_array('directory_index', 'page');
            foreach ($directory_index as $index) {
                $page = $page_dao->select_by_page_path($pathname . $index);
                if ($page !== null) {
                    $pathname .= $index;
                    break;
                }
            }
        }
        // 未登録ファイルの場合、管理対象かどうか確認してファイルを登録処理を実行する
        $page_logic = new Page_Logic();
        if ($page === null && $this->is_target_file($pathname)) {
            list($page_list, $delete_list) = $page_logic->get_sync_list();
            // DBに登録
            $page_logic->insert_page_list($page_list);
            // ページ情報を取得
            $page = $page_dao->select_by_page_path($pathname);
        }

        // フォルダ情報取得
        // /ディレクトリ/ 終わりだと、dirnameの階層が変わってしまうので、'.'を追加
        $path_parts = pathinfo($pathname . '.');
        $dirname = ($path_parts['dirname'] === DIRECTORY_SEPARATOR) ? '/' : $path_parts['dirname'] . '/';
        $rows = $page_logic->select_page_list($dirname);

        // 選択ファイル確認
        foreach ($rows as &$row) {
            if ($row['page_path'] === $pathname) {
                $row['selected'] = true;
            }
        }

        return array($page, $rows);
    }

    /**
     * 管理対象ファイルかどうかを判定する
     * @param string $pathname
     * @return boolean
     */
    private function is_target_file($pathname)
    {
        // 除外ディレクトリ配下ではないことを確認
        $exclude_dir = Me_Config::get_array('exclude_dir', 'page');
        foreach ($exclude_dir as $ex_dir) {
            if (strpos($pathname, $ex_dir) === 0) {
                // 除外ディレクトリ配下の場合は、管理対象ではないと判定
                return false;
            }
        }

        // 拡張子確認
        $path_parts = pathinfo($pathname);
        $extension_list = Me_Config::get_array('extensions', 'page');
        $file_pattern = '/^(' . implode('|', $extension_list) . ')$/i';
        if (empty($extension_list) || !preg_match($file_pattern, $path_parts['extension'])) {
            // extensionsの設定値がない場合、及び拡張子に一致しない場合は登録しない
            return false;
        }
        return true;
    }
}