<?php
/**
 * Metag Attr
 * @package     Metag
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Metag_Attr extends Metag
{
    /**
     * タグタイプ
     * @var string
     */
    public $node_type = 'attr';
    /**
     * 表示名
     * @var string
     */
    public $display_name = '属性 (attr)';
    /**
     * ブロック要素かどうか
     * @var boolean
     */
    public $is_block = false;
    /**
     * 対象属性
     * @var string
     */
    public $target;
    /**
     * node_valueのプレフィックス
     * @var string
     */
    public $prefix;
    /**
     * node_valueのサフィックス
     * @var string
     */
    public $suffix;
    /**
     * 出力条件( first / last / odd / even / self )
     * @var string
     */
    public $filter;
    /**
     * 属性リスト
     * @var array
     */
    public $attr_list = array(
        'id' => 'id',
        'target' => 'target',
        'prefix' => 'prefix',
        'suffix' => 'suffix',
        'filter' => 'filter',
        'initial_value' => 'initial_value',
        'ref_id' => 'ref_id',
        'ref_page_id' => 'ref_page_id',
    );

    /**
     * 編集viewの開始タグ出力
     * @param array $row
     */
    public function edit_view($row)
    {
        $child = $this->get_child_node();
        echo $child->node_value;
    }

    /**
     * 出力
     */
    public function output($row, $edit_mode = false)
    {
        $child = $this->get_child_node();
        $replace_flag = true;

        // filter
        if (strlen($this->filter) && strlen($this->display_count)) {
            $replace_flag = false;
            switch ($this->filter) {
                case 'first': // 最初
                    if ($this->display_count === 1) {
                        $replace_flag = true;
                    }
                    break;
                case 'last': // 最後
                    if ($this->display_count === $this->repeat_count) {
                        $replace_flag = true;
                    }
                    break;
                case 'odd': // 奇数
                    if (($this->display_count % 2) === 1) {
                        $replace_flag = true;
                    }
                    break;
                case 'even': // 偶数
                    if (($this->display_count % 2) === 0) {
                        $replace_flag = true;
                    }
                    break;
                case 'self': // 自身ページ
                    if (($row['_self_page_id'] === $row['_page_id'])) {
                        $replace_flag = true;
                    }
                    break;
                default:
                    break;
            }
        }

        if ($replace_flag) {
            $ret = preg_replace(
                '/' . $this->target . '=(""|".*?[^\\\\]"|\'\'|\'.*?[^\\\\]\')/',
                $this->target . '="' . Me_Util::html_escape($this->prefix . $row['node_value'] . $this->suffix) . '"',
                $child->node_value);
        } else {
            $ret = $child->node_value;
        }
        return $ret;
    }

    /**
     * タグ編集
     */
    public function set_post_data()
    {
        $mapping = $this->attr_list;
        $mapping['selected_text'] = 'selected_text';
        Me_Util::post_data_to_object($this, $mapping);
    }
}