<?php
/**
 * Metag Dao
 * @package     Metag
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Metag_Dao extends Me_Dao
{
    /**
     * テーブル名
     * @var string
     */
    public $table_name = 'metag';
    /**
     * Entityクラス名
     * @var stirng
     */
    public $class_name = 'Metag';
    /**
     * カラム定義配列
     * @var array
     */
    public $column_list = array(
        '_id',
        'page_id',
        'repeat_id',
        'repeat_no',
        'id',
        'node_value',
        'url',
        'attr',
    );
    /**
     * リビジョン登録
     * @var boolean
     */
    public $register_revision = false;
    /**
     * リビジョンID
     */
    public $revision_id;
    /**
     * MEタグリビジョンDAO
     * @var Metag_Revision_Dao
     */
    public $metag_revision_dao;

    /**
     * コンストラクタ
     */
    public function __construct()
    {

    }

    /**
     * ページのMEタグ情報を取得
     * @param string $page_id
     * @return array $data
     */
    public function get_page_data($page_id = null)
    {
        $data = array();
        if ($page_id === null) {
            return $data;
        }

        // MEタグ情報取得
        $metag_rows = $this->select_by_page_id($page_id);

        // MEタグの値をdataにセット
        foreach ($metag_rows as $row) {
            if (strlen($row['repeat_id']) > 0) {
                // repeat内のタグ
                $data[$row['repeat_id']][$row['repeat_no']][$row['id']] = $row;
            } else {
                $data[$row['id']] = $row;
            }
        }
        foreach ($data as &$data_row) {
            ksort($data_row);
        }
        return $data;
    }

    /**
     * ページIDでレコードを取得
     * @param int $page_id
     * @param boolean $public_only
     * @return array $rows
     */
    public function select_by_page_id($page_id, $public_only = false)
    {
        if (empty($page_id)) {
            return array();
        }

        $query = <<< QUERY
select m._id, m.page_id, m.id, m.node_value, m.url, m.attr, m.repeat_id, m.repeat_no, p.page_path as ref_link_path
from metag as m
left outer join page as p on (m.page_id = p._id)
where m.page_id = 1
and p.public_flag = 1
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $page_id);
        if (!$public_only) {
            $sql->remove('and');
        }
        $rows = $sql->exec();
        foreach ($rows as &$row) {
            if (!empty($row['attr'])) {
                $row['attr'] = unserialize($row['attr']);
            }
            if (strlen(SITE_PATH) > 0 && !empty($row['ref_link_path'])) {
                $row['ref_link_path'] = SITE_PATH . $row['ref_link_path'];
            }
        }
        return $rows;
    }

    /**
     * テンプレートIDでレコードを取得
     * 公開しているページのみ取得する
     * @param stirng $template_id
     * @param string $limit
     * @param string $offset
     * @param string $sort_by
     * @param string $sort_order
     * @return array $rows
     */
    public function select_by_template_id($template_id, $limit = null, $offset = null, $sort_by = Me_Sql::CREATE_DATE, $sort_order = Me_Sql::DESC)
    {
        if (empty($template_id)) {
            return array();
        }
        $template_id_list = array_map('trim', explode(',', $template_id));

        $query = <<< QUERY
select page_id, id, node_value, url, attr, repeat_id, repeat_no, p.page_path as ref_link_path, p.template_id, p.create_date, p.update_date, p.title
from metag as m
join
(select _id, template_id, page_path, create_date, update_date, title from page
where template_id in ( '1' ) and public_flag = 1
/*1*/ order by create_date desc , _id desc
) as p on (m.page_id = p._id)
order by create_date desc , page_id desc
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 4, $template_id_list);

        if ($sort_by === Me_Sql::UPDATE_DATE) {
            $sql->replace('/*1*/', 3, Me_Sql::UPDATE_DATE);
            $sql->replace('order', 2, Me_Sql::UPDATE_DATE);
        }
        if ($sort_order === Me_Sql::ASC) {
            $sql->replace('/*1*/', 4, Me_Sql::ASC);
            $sql->replace('order', 3, Me_Sql::ASC);
        }
        $rows = $sql->exec();

        $page_list = array();
        $page_id = null;
        $repeat_no = 0;
        // page_id毎の配列にまとめる
        foreach ($rows as $row) {
            $ref_link_path = (strlen(SITE_PATH) > 0 && !empty($row['ref_link_path'])) ? SITE_PATH . $row['ref_link_path'] : $row['ref_link_path'];
            if ($page_id != $row['page_id']) {
                $page_id = $row['page_id'];
                $repeat_no++;

                // 予約ID情報
                $page_list[$repeat_no]['_path'] = array('node_value' => $row['ref_link_path']);
                $page_list[$repeat_no]['_page_id'] = array('node_value' => $row['page_id']);
                $page_list[$repeat_no]['_template_id'] = array('node_value' => $row['template_id']);
                $page_list[$repeat_no]['_create_date'] = array('node_value' => date(Me_Config::SYSTEM_DATE_FORMAT, $row['create_date']));
                $page_list[$repeat_no]['_update_date'] = array('node_value' => date(Me_Config::SYSTEM_DATE_FORMAT, $row['update_date']));
                $page_list[$repeat_no]['_now'] = array('node_value' => NOW);
                $page_list[$repeat_no]['_title'] = array('node_value' => $row['title'], 'ref_link_path' => $ref_link_path);
            }
            if (!empty($row['attr'])) {
                $row['attr'] = unserialize($row['attr']);
            }
            if (strlen(SITE_PATH) > 0 && !empty($row['ref_link_path'])) {
                $row['ref_link_path'] = SITE_PATH . $row['ref_link_path'];
            }
            $page_list[$repeat_no][$row['id']] = $row;
        }
        return $page_list;
    }

    /**
     * ページID指定でレコードを削除
     * @param string $page_id
     * @return boolean
     */
    public function delete_by_page_id($page_id)
    {
        if (strlen($page_id) <= 0) {
            return false;
        }

        $query = <<< QUERY
delete from metag
where page_id = 1
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $page_id);
        $sql->exec();
        return true;
    }

    /**
     * テンプレートID指定でレコードを削除
     * @param string $template_id
     * @return boolean
     */
    public function delete_by_template_id($template_id)
    {
        if (strlen($template_id) <= 0) {
            return false;
        }

        $query = <<< QUERY
delete from metag 
where page_id in ( select _id from page where 
template_id = 1 )
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('template_id', 2, $template_id);
        $sql->exec();
        return true;
    }

    /**
     * 一括登録
     * @param Page_Form $form
     * @param integer $revision_id
     * @return boolean
     */
    public function register_array($form, $revision_id = null)
    {
        if ($revision_id !== null) {
            $this->register_revision = true;
            $this->revision_id = $revision_id;
            $this->metag_revision_dao = new Metag_Revision_Dao();
        }
        
        $image_logic = new Content_Image_Logic();
        $file_logic = new Content_File_Logic();

        // 旧レコードは一旦削除
        $this->delete_by_page_id($form->_id);

        foreach ($form->data as $key => $row) {
            $metag = $form->document->get_element_by_id($key);
            if ($metag === null || $metag->repeat_id != $row['repeat_id'] || strlen($metag->ref_id) > 0 || substr($metag->id, 0, 1) === '_') {
                // repeat_idが一致しない場合（空同士一致でも良い）はDBを更新しない
                // あるいは、参照元の場合。
                continue;
            }
            if ($metag->node_type === 'repeat') {
                // inside of repeat tag
                $new_repeat_no = 1;
                foreach ($row as $repeat_no => $repeat_list) {
                    $exist = false;
                    foreach ($repeat_list as $repeat_inside_id => $repeat_inside_row) {
                        $repeat_inside_metag = $form->document->get_element_by_id($repeat_inside_id);
                        // MEタグがテンプレート上で削除された場合
                        if ($repeat_inside_metag === null) {
                            continue;
                        }
                        $repeat_inside_row['repeat_no'] = $new_repeat_no;
                        $repeat_inside_row['page_id'] = $form->_id;
                        // repeat内は値が無い場合は登録しない。
                        if ($repeat_inside_metag->exist_node_value($repeat_inside_row)) {
                            if ($repeat_inside_metag->node_type === 'img') {
                                if ($this->move_img($repeat_inside_row, $repeat_inside_metag, $form->new_page_path)) {
                                    $image_logic->register($repeat_inside_row['node_value'], $form->_id, $form->page_path, $form->user_id, $form->user_name);
                                }
                                $form->unset_unlink_files($repeat_inside_row['node_value']);
                            } else if ($repeat_inside_metag->node_type === 'file') {
                                if ($this->move_file($repeat_inside_row, $repeat_inside_metag, $form->new_page_path)) {
                                    $file_logic->register($repeat_inside_row['node_value'], $form->_id, $form->page_path, $form->user_id, $form->user_name);
                                }
                                $form->unset_unlink_files($repeat_inside_row['node_value']);
                            } 
                            $this->register_row($repeat_inside_row);
                            $exist = true;
                        }
                    }
                    if ($exist) {
                        $new_repeat_no++;
                    }
                }
            } else {
                $row['id'] = $key;
                $row['page_id'] = $form->_id;
                if ($metag->node_type === 'img') {
                    if ($this->move_img($row, $metag, $form->new_page_path)) {
                        $image_logic->register($row['node_value'], $form->_id, $form->page_path, $form->user_id, $form->user_name);
                    }
                    $form->unset_unlink_files($row['node_value']);
                } else if ($metag->node_type === 'file') {
                    if ($this->move_file($row, $metag, $form->new_page_path)) {
                        $file_logic->register($row['node_value'], $form->_id, $form->page_path, $form->user_id, $form->user_name);
                    }
                    $form->unset_unlink_files($row['node_value']);
                }
                $this->register_row($row);
            }
        }
        return true;
    }

    public function delete_page($page_id)
    {
        if (empty($page_id) || !is_numeric($page_id)) {
            // page_idがなければfalseを返す。
            return false;
        }

        $query = <<< QUERY
delete from metag
where page_id = 0
QUERY;

        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind('where', 3, $page_id);
        $sql->exec();
        return true;
    }

    /**
     * レコード登録
     * @param array $row
     */
    private function register_row($row)
    {
        $metag = (object)$row;
        if (is_array($metag->attr) && !empty($metag->attr)) {
            $metag->attr = serialize($metag->attr);
        } else {
            $metag->attr = '';
        }
        $metag->_id = null;
        
        // リストタグ対策
        if (is_array($metag->node_value)) {
            $metag->node_value = serialize($metag->node_value);
        }
        $this->insert($metag);
        
        // リビジョン登録
        if ($this->register_revision) {
            $metag->_id = null;
            $metag->revision_id = $this->revision_id;
            $this->metag_revision_dao->insert($metag);
        }
    }

    /**
     * 画像ファイルを一時ディレクトリからUPLOADディレクトリに移動する
     * @param array $row
     * @param object $metag
     * @param string $new_page_path
     * @return boolean
     */
    private function move_img(&$row, $metag, $new_page_path)
    {
        // 一時ディレクトリ内にあることを確認
        if (strpos($row['node_value'], UPLOAD_PATH . Page_Form::TEMPORARY_DIRECTORY . '/') !== 0) {
            return false;
        }
        
        // 共通のアップロードファイル保存ディレクトリ
        $save_dir_path = UPLOAD_PATH;
        $save_dir_root = UPLOAD_ROOT;

        // ページと同じディレクトリ内に移動する場合
        if (Me_Config::get('per_upload_dir') !== null) {
            // 保存ディレクトリパスを作成
            $new_page_dir = (dirname($new_page_path) === '/') ? '/' : dirname($new_page_path) . '/';
            $save_dir_path = SITE_PATH . $new_page_dir . Me_Config::get('per_upload_dir');
            $save_dir_root = DOCUMENT_ROOT . $save_dir_path;
            // 保存ディレクトリが無ければ作成
            if (!file_exists($save_dir_root)) {
                Me_Util::mkdir($save_dir_root);
            }
        }

        $old_file_name = $default_old_file_name = basename($row['node_value']);
        $i = 1;
        while (true) {
            if ($i > 1) {
                $old_file_name = preg_replace('/\.[^\.]*$/', '-' . $i . '$0', $default_old_file_name);
            }
            $old_file_path = UPLOAD_ROOT . Page_Form::TEMPORARY_DIRECTORY . '/' . $old_file_name;
            if (file_exists($old_file_path) && is_readable($old_file_path)) {
                $new_file_path = $save_dir_root . '/' . $old_file_name;
                // ファイルを移動
                if (!rename($old_file_path, $new_file_path)) {
                    break;
                }
            } else {
                break;
            }
            $i++;
        }
        $row['node_value'] = $save_dir_path . '/' . $default_old_file_name;
        if (strlen($metag->self_link) > 0) {
            $row['url'] = preg_replace('/\.[^\.]*$/', '-' . $metag->self_link . '$0', $row['node_value']);
        }
        return true;
    }

    /**
     * ファイルを一時ディレクトリからUPLOADディレクトリに移動する
     * @param array $row
     * @param object $metag
     * @param string $new_page_path
     * @return boolean
     */
    private function move_file(&$row, $metag, $new_page_path)
    {
        // 一時ディレクトリ内にあることを確認
        if (strpos($row['node_value'], UPLOAD_PATH . Page_Form::TEMPORARY_DIRECTORY . '/' . $row['attr']['dirname'] . '/') !== 0) {
            return false;
        }

        $old_file_name = basename($row['node_value']);
        $old_dir_path = UPLOAD_ROOT . Page_Form::TEMPORARY_DIRECTORY . '/' . $row['attr']['dirname'] . '/';
        $old_file_path = $old_dir_path . $old_file_name;
        // 共通のアップロードファイル保存ディレクトリ
        $files_path = UPLOAD_PATH . Page_Form::FILES_DIRECTORY . '/';
        $files_root = UPLOAD_ROOT . Page_Form::FILES_DIRECTORY . '/';
        
        // ページと同じディレクトリ内に移動する場合
        if (Me_Config::get('per_upload_dir') !== null) {
            $files_path = SITE_PATH . dirname($new_page_path) . '/' . Me_Config::get('per_upload_dir') . Page_Form::FILES_DIRECTORY . '/';
            $files_root = DOCUMENT_ROOT . $files_path;
        }

        if (!is_dir($files_root)) {
            Me_Util::mkdir($files_root, 0755, true);
        }

        if (is_dir($old_dir_path)) {
            // 環境により、phpのrename関数がディレクトリの移動に失敗するため、ディレクトリを作り直す。
            $new_dir_path = $files_root . $row['attr']['dirname'] . '/' ;
            $new_file_path = $new_dir_path . $old_file_name;
            // ディレクトリを作成
            Me_Util::mkdir($new_dir_path, 0755, true);
            // ファイルを移動
            Me_Util::rename_file($old_file_path, $new_file_path);
            
            // 一時ディレクトリ内のディレクトリを削除
            if (is_writable($old_dir_path)) {
                // ディレクトリを削除
                rmdir($old_dir_path);
            }
        }
        $row['node_value'] = $files_path . $row['attr']['dirname'] . '/'  . $old_file_name;
        
        return true;
    }
}